package com.tmerclub.cloud.cuerecycle.manager;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.tmerclub.cloud.api.coupon.feign.CouponFeignClient;
import com.tmerclub.cloud.api.delivery.feign.DeliveryFeignClient;
import com.tmerclub.cloud.api.order.feign.OrderAddrFeignClient;
import com.tmerclub.cloud.api.order.feign.OrderFeignClient;
import com.tmerclub.cloud.api.user.feign.UserAddrFeignClient;
import com.tmerclub.cloud.common.exception.LuckException;
import com.tmerclub.cloud.common.local.constant.CueOrderStatus;
import com.tmerclub.cloud.common.local.dto.SaveOrderToMallDTO;
import com.tmerclub.cloud.common.order.vo.OrderAddrVO;
import com.tmerclub.cloud.common.order.vo.UserAddrVO;
import com.tmerclub.cloud.common.response.ServerResponseEntity;
import com.tmerclub.cloud.cuerecycle.mapper.CueOrderFlawMapper;
import com.tmerclub.cloud.cuerecycle.mapper.CueOrderMapper;
import com.tmerclub.cloud.cuerecycle.model.CueOrder;
import com.tmerclub.cloud.cuerecycle.model.CueOrderFlaw;
import com.tmerclub.cloud.cuerecycle.model.dto.CreateOrderDTO;
import com.tmerclub.cloud.cuerecycle.model.dto.CreateOrderFlawDTO;
import com.tmerclub.cloud.cuerecycle.model.dto.CueRefuseOrderDTO;
import com.tmerclub.cloud.cuerecycle.model.vo.CueOrderVO;
import com.tmerclub.cloud.cuerecycle.service.CueFlawService;
import jakarta.annotation.Resource;
import org.apache.dubbo.config.annotation.DubboReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Objects;

/**
 * 确认订单管理器
 *
 * @author: frank
 * @create: 2025-04-15
 **/
@Component
public class CreateOrderManager {
    private final Logger log = LoggerFactory.getLogger(CreateOrderManager.class);

    @Resource
    CueFlawService cueFlawService;
    @Resource
    CueOrderMapper cueOrderMapper;
    @Resource
    CueOrderFlawMapper cueOrderFlawMapper;
    @DubboReference
    CouponFeignClient couponFeignClient;
    @DubboReference
    private OrderFeignClient orderFeignClient;
    @DubboReference
    DeliveryFeignClient deliveryFeignClient;
    @DubboReference
    UserAddrFeignClient userAddrFeignClient;
    @DubboReference
    OrderAddrFeignClient orderAddrFeignClient;

    /**
     * 保存订单信息
     *
     * @param createOrderDTO 订单信息
     * @return 订单id
     */
    public Long saveOrder(CreateOrderDTO createOrderDTO) {
        // 保存订单信息
        CueOrder cueOrder = BeanUtil.toBean(createOrderDTO, CueOrder.class);
        cueOrder.setOrderStatus(CueOrderStatus.PENDING_AUDIT.value());
        cueOrder.setCreateTime(DateUtil.date());
        cueOrderMapper.save(cueOrder);
        return cueOrder.getOrderId();
    }

    /**
     * 添加瑕疵项
     *
     * @param createOrderDTO 订单信息
     */
    public Long addFlawItem(Long orderId, CreateOrderDTO createOrderDTO) {
        // 循环插入瑕疵信息并累计选择金额
        Long flawPrice = 0L;
        for (CreateOrderFlawDTO createOrderFlawDTO : createOrderDTO.getFlawList()) {
            if (createOrderFlawDTO.getIsFlaw() == 1) {
                // 累加瑕疵金额
                flawPrice += createOrderFlawDTO.getFlawPrice();
            }
            CueOrderFlaw cueOrderFlaw = BeanUtil.toBean(createOrderFlawDTO, CueOrderFlaw.class);
            cueOrderFlaw.setOrderId(orderId);
            cueOrderFlawMapper.save(cueOrderFlaw);
        }
        return flawPrice;
    }

    /**
     * 保存订单到mallOrder
     */
    public void saveOrderToMall(Long orderId, Long estimatedAmount, CreateOrderDTO createOrderDTO) {
        SaveOrderToMallDTO saveOrderToMallDTO = new SaveOrderToMallDTO();
        saveOrderToMallDTO.setUserId(createOrderDTO.getUserId());
        saveOrderToMallDTO.setEstimatedAmount(estimatedAmount);
        saveOrderToMallDTO.setProductName(createOrderDTO.getProductName());
        saveOrderToMallDTO.setProductImages(createOrderDTO.getProductImages());
        Long mallOrderId = orderFeignClient.saveOrderToMall(saveOrderToMallDTO);
        CueOrder cueOrder = new CueOrder();
        cueOrder.setOrderId(orderId);
        cueOrder.setMallOrderId(mallOrderId);
        cueOrder.setEstimatedAmount(estimatedAmount);
        cueOrderMapper.update(cueOrder);
    }

    /**
     * 修改mallOrder订单
     *
     * @param mallOrderId mallOrderId
     * @param orderStatus 状态
     */
    public void updateOrderMall(Long mallOrderId, Integer orderStatus) {
        orderFeignClient.updateOrderMall(mallOrderId, orderStatus);
    }

    public void agreeOrder(CreateOrderDTO createOrderDTO) {
        // 查询订单信息
        CueOrderVO cueOrderVO = cueOrderMapper.getById(createOrderDTO.getOrderId());
        if (ObjectUtil.isNull(cueOrderVO)) {
            throw new LuckException("订单不存在,请检查!");
        }
        // 判断状态是否允许修改
        if (!Objects.equals(cueOrderVO.getOrderStatus(), CueOrderStatus.EVALUATION_SUCCESS.value())) {
            throw new LuckException("状态已变更,请检查!");
        }
        CueOrder cueOrder = new CueOrder();
        cueOrder.setOrderId(createOrderDTO.getOrderId());
        cueOrder.setOrderType(createOrderDTO.getOrderType());
        cueOrder.setOrderStatus(CueOrderStatus.EVALUATION_SUCCESS_AND_PAY.value());
        cueOrder.setUpdateTime(DateUtil.date());
        cueOrder.setFinallyTime(DateUtil.date());
        cueOrder.setIsPayed(1);
        // 根据回收类型进行不同操作
        switch (createOrderDTO.getOrderType()) {
            case 1:
                // 回收
                cueOrder.setPayType(1);
                break;
            case 2:
                // 置换
                cueOrder.setPayType(2);
                break;
            case 3:
                // 寄售
                cueOrder.setPayType(1);
                break;
            default:
                throw new LuckException("订单类型错误,请检查!");
        }
        // 根据支付方式进行不同操作
        switch (cueOrder.getPayType()) {
            case 1:
                // 余额支付 直接向用户账户打款
                break;
            case 2:
                // 代金卷支付
                couponFeignClient.sendUserCoupon(cueOrderVO.getUserId(), cueOrderVO.getVoucherAmount());
                break;
            default:
                throw new LuckException("支付方式错误,请检查!");
        }
        cueOrderMapper.update(cueOrder);
        this.updateOrderMall(cueOrderVO.getMallOrderId(), CueOrderStatus.EVALUATION_FAIL.value());
    }

    public void refuseOrder(CueRefuseOrderDTO cueRefuseOrderDTO) {

        Long orderId = cueRefuseOrderDTO.getOrderId();
        // 查询订单信息
        CueOrderVO cueOrderVO = cueOrderMapper.getById(orderId);
        if (ObjectUtil.isNull(cueOrderVO)) {
            throw new LuckException("订单不存在,请检查!");
        }
        // 两种情况会导致退款 5不合格退款 6价格不满意退款
        // 检查当前状态
        if (!Objects.equals(cueOrderVO.getOrderStatus(), CueOrderStatus.EVALUATION_SUCCESS.value()) || !Objects.equals(cueOrderVO.getOrderStatus(), CueOrderStatus.RECEIPT_EVALUATION.value())) {
            throw new LuckException("状态已变更,请检查!");
        }

        CueOrder cueOrder = new CueOrder();
        cueOrder.setOrderId(orderId);
        // 需要向order_addr插入一条数据
        ServerResponseEntity<UserAddrVO> userAddrVO = userAddrFeignClient.cueGetUserAddrByAddrId(cueRefuseOrderDTO.getAddrId(), cueOrderVO.getUserId());
        if (userAddrVO.isSuccess()) {
            // 将地址信息插入到order_addr中
            OrderAddrVO orderAddrVO = BeanUtil.toBean(userAddrVO, OrderAddrVO.class);
            orderAddrVO.setUserId(cueOrderVO.getUserId());
            ServerResponseEntity<Long> longServerResponseEntity = orderAddrFeignClient.addCueAddr(orderAddrVO);
            if (longServerResponseEntity.isSuccess()) {
                cueOrder.setOrderAddrId(longServerResponseEntity.getData());
            }
        }
        if(Objects.equals(cueOrderVO.getOrderStatus(), CueOrderStatus.EVALUATION_SUCCESS.value())){
            cueOrder.setOrderStatus(CueOrderStatus.EVALUATION_FAIL.value());
        }else{
            cueOrder.setOrderStatus(CueOrderStatus.EVALUATION_FAIL.value());
        }

        cueOrder.setUpdateTime(DateUtil.date());
        cueOrderMapper.update(cueOrder);
        this.updateOrderMall(cueOrderVO.getMallOrderId(), CueOrderStatus.EVALUATION_FAIL.value());
    }
}